use actix_web::{App, HttpServer};
use sea_orm::DatabaseConnection;

use crate::Opts;

mod urls;

mod controller;
mod model;
mod view;

#[derive(Debug, Clone)]
struct AppState {
    database: DatabaseConnection,
}

pub(crate) async fn run(
    database: DatabaseConnection,
    opts: Opts,
) -> Result<(), Box<dyn std::error::Error>> {
    let data = actix_web::web::Data::new(AppState { database: database });

    let server =
        HttpServer::new(move || App::new().configure(urls::configure).app_data(data.clone()))
            .bind((opts.address, opts.port))?;

    let result = server.run().await?;

    Ok(result)
}
